#include <loudmouth/loudmouth.h>
#include <string.h>
#include "kf.h"
#include "archive.h"
#include "jabber.h"
#include "jabber_msg.h"
#include "filter.h"
#include "message.h"

void kf_message_log (KfJabberMessage *msg, gboolean yours);

/* Odbiornik wiadomoci */
LmHandlerResult kf_jabber_message_hendel    (LmMessageHandler *handler,
                                             LmConnection *connection,
                                             LmMessage *message,
                                             gpointer user_data)
{
	LmMessageNode *node;
	const gchar *from;
	KfJabberMessage *msg;
	LmMessageSubType subtype;


	subtype = lm_message_get_sub_type (message);
	from = lm_message_node_get_attribute (message->node, "from");
	if (kf_filter_is_blocked (from)) {
		foo_debug ("Messages from '%s' are blocked!!!\n", from);
		return LM_HANDLER_RESULT_REMOVE_MESSAGE;
	}
/*	node = lm_message_node_get_child (message->node, "subject");
	subject = node?node->value:NULL;
	node = lm_message_node_get_child (message->node, "body");
	body = node?node->value:NULL;
*/

//	g_printerr ("> %s\n",lm_message_node_to_string (message->node));

	msg = kf_jabber_message_new ();
	msg->type = (subtype == LM_MESSAGE_SUB_TYPE_CHAT)?KF_JABBER_MESSAGE_TYPE_CHAT:KF_JABBER_MESSAGE_TYPE_NORMAL;
	msg->from = g_strdup (from);

	msg->special_me = FALSE;
//	msg->subject = g_strdup (subject);
//	msg->body = g_strdup (body);

	for (node = message->node->children; node; node = node->next) {
		if (!strcmp (node->name, "subject")) {
			msg->subject = g_strdup (node->value);
		} else if (!strcmp (node->name, "body")) {
			msg->body = g_strdup (node->value);

			if (strstr (node->value, "/me") == node->value) {
//				g_print ("meeeee....\n");
				msg->special_me = TRUE;
			}
		} else if (!strcmp (node->name, "x")) {
			const gchar *xmlns;

			xmlns = lm_message_node_get_attribute (node, "xmlns");
			if (!strcmp (xmlns, "jabber:x:delay")) {
				const gchar *stamp;

				stamp = lm_message_node_get_attribute (node, "stamp");
//				g_print ("\n---------------------------> %s\n\n", stamp);
				msg->stamp = x_stamp_to_time (stamp);
			}
		}
	}

	kf_archive_msg (message, FALSE);

	kf_message_process (msg);

	kf_jabber_message_unref (msg);
	
//	kf_gui_message_normal_display (from, subject, body);
	return LM_HANDLER_RESULT_REMOVE_MESSAGE;
}

/* Wysanie wiadomoci */
gboolean kf_jabber_message_send (KfJabberMessage *jmsg) {
	LmMessage *msg;
	extern LmConnection *kf_jabber_connection;
	GError *error;
	
	kf_message_log (jmsg, TRUE);

	if (jmsg->type == KF_JABBER_MESSAGE_TYPE_CHAT) {
	//	lm_message_node_set_attribute (msg->node, "type", "chat");
		msg = lm_message_new_with_sub_type (jmsg->to,
				LM_MESSAGE_TYPE_MESSAGE,
				LM_MESSAGE_SUB_TYPE_CHAT);
	} else {
		msg = lm_message_new_with_sub_type (jmsg->to,
				LM_MESSAGE_TYPE_MESSAGE,
				LM_MESSAGE_SUB_TYPE_NORMAL);
	}
	
	if (jmsg->subject)
		lm_message_node_add_child (msg->node, "subject", jmsg->subject);
	if (jmsg->body)
		lm_message_node_add_child (msg->node, "body", jmsg->body);


	if (!lm_connection_send (kf_jabber_connection, msg, &error)) {
		g_error ("Nie mona wysa wiadomoci: %s\n", error->message);
	}

	kf_archive_msg (msg, TRUE);
	lm_message_unref (msg);	/* Mam nadziej e to zwalnia pami */
	return TRUE;
}


